%module(directors="1") wali

%include "typemaps.i"
%include "std_string.i"
%include "std_pair.i"
%define SWIG_EXPORT_ITERATOR_METHODS
%enddef
%include "std_set.i"
%include "std_vector.i"

%{
/* Util stuff */
#include "wali_src/Source/wali/ref_ptr.hpp"
#include "wali_src/Source/wali/Countable.hpp"
#include "wali_src/Source/wali/IMarkable.hpp"
#include "wali_src/Source/wali/Printable.hpp"
#include "wali_src/Source/wali/Key.hpp"
#include "wali_src/Source/wali/regex/Regex.hpp"

/* WFA Util stuff */
#include "wali_src/Source/wali/wfa/TransFunctor.hpp"
/* WFA */
#include "wali_src/Source/wali/wfa/WFA.hpp"

/* WPDS util stuff */
#include "wali_src/Source/wali/wpds/Rule.hpp"
#include "wali_src/Source/wali/wpds/GenKeySource.hpp"

/* WPDS's */
#include "wali_src/Source/wali/wpds/WPDS.hpp"
#include "wali_src/Source/wali/wpds/ewpds/EWPDS.hpp"
#include "wali_src/Source/wali/wpds/fwpds/FWPDS.hpp"

/* Examples */
#include "wali_src/Examples/Reach/Reach.hpp"

/* Include implementation inline (hack hack)*/
#include "wali_src/Examples/Reach/Reach.cpp"
#include "ConstPropagation/ConstDom.cpp"

#include <sstream>
#include <utility>

using namespace wali;
using namespace wali::wpds;
using namespace wali::wfa;
%}

%feature("ref")   SemElem "$this->count++;"
%feature("unref") SemElem "$this->count--;"

/*
%feature("director") SemElem;
%feature("director") SemElemPtr;
*/

/* Ignore ostream users */
%ignore wali::SemElem::print;
%ignore wali::SemElem::marshall;
%ignore wali::Printable::print;

/* Guards */
%ignore wali_wpds_WPDS_GUARD;
%ignore wali_KEY_GUARD;
%ignore wali_REF_PTR_GUARD;
%ignore wali_SEM_ELEM_GUARD;


/* Let's just grab the original header file here */
%include "wali_src/Source/wali/ref_ptr.hpp"
%include "wali_src/Source/wali/Printable.hpp"
%rename(__str__) wali::Printable::to_string;
%ignore wali::Printable::toString;

%include "wali_src/Source/wali/Key.hpp"

/* Helper function for getting epsilon char */
wali::Key getEpsilonKey();
%{
wali::Key getEpsilonKey() {
    return wali::getKey("*");
}
%}

%include "wali_src/Source/wali/KeySource.hpp"

%include "wali_src/Source/wali/wpds/GenKeySource.hpp"



%include "wali_src/Source/wali/SemElem.hpp"

%template(SemElemPtr) wali::ref_ptr<SemElem>;

%extend wali::SemElem {
    PyObject* __str__() {
        std::ostringstream outs;
        $self->print(outs);

        return PyString_FromString(outs.str().c_str());
    }
}

%pythonprepend wali::SemElem::combine(SemElem* se) %{
        #Deref smart pointer if needed
        if hasattr(args[1], '__deref__'):
            tmp = list(args)
            tmp[1] = tmp[1].__deref__()
            args = tuple(tmp)
%}

%pythonprepend wali::SemElem::extend(SemElem* se) %{
        #Deref smart pointer if needed
        if hasattr(args[1], '__deref__'):
            tmp = list(args)
            tmp[1] = tmp[1].__deref__()
            args = tuple(tmp)
%}


/* ==================== WFA ====================== */

/*%import "wali_src/Source/wali/wfa/TransFunctor.hpp"*/ /* This messes stuff up :-/ */

/*%include <pair.h>*/
%template(KeyPair) std::pair< Key,Key >;

%include "wali_src/Source/wali/hm_hash.hpp"
%include "wali_src/Source/wali/HashMap.hpp"
%template(KeyPairMap) wali::HashMap< KeyPair, TransSet, hm_hash< KeyPair >, hm_equal< KeyPair > >;


#define ATTR_UNUSED
%import "wali_src/Source/wali/IMarkable.hpp"


%rename(fromState) wali::wfa::Trans::from;
%rename(toState) wali::wfa::Trans::to;
%include "wali_src/Source/wali/wfa/Trans.hpp"

%traits_swigtype(ITrans);
%fragment(SWIG_Traits_frag(ITrans));

%template(TransSetImpl) std::set< ITrans*,ITransLT >;
/*%template(TransSetIterator) std::set< ITrans*,ITransLT >::iterator;*/


%include "wali_src/Source/wali/wfa/TransSet.hpp"

%template(TransVector) std::vector<wali::wfa::Trans*>;

%extend wali::wfa::TransSet {
    std::vector< wali::wfa::Trans* > asList() {
        std::vector< Trans* > toRet;
        //std::cerr << "asList called with " << $self << std::endl;

        TransSet::iterator it = $self->begin();
        TransSet::iterator itend = $self->end();
        for( ; it != itend ; it++ ) {
          toRet.push_back( static_cast<wali::wfa::Trans*>(*it) );
        }

        return toRet;
    }
}

/*%feature("shadow") wali::wfa::TransSet::__iter__() %{
def __iter__(*args):
    pass
    #do something before
    #do something after
%}*/

/*%feature("shadow") wali::wfa::TransSet::find()
%{
def find(*args):
    pass
    #do something before
    #do something after
%}*/


%include "wali_src/Source/wali/wfa/WFA.hpp"

%extend wali::wfa::WFA {
    PyObject* __str__() {
        std::ostringstream outs;
        $self->print(outs);

        return PyString_FromString(outs.str().c_str());
    }

    PyObject* print_dot() {
        std::ostringstream outs;
        $self->print_dot(outs);

        return PyString_FromString(outs.str().c_str());
    }


}



/* ==================== WPDS ====================== */

%include "wali_src/Source/wali/wpds/WPDS.hpp"

%extend wali::wpds::WPDS {
    PyObject* __str__() {
        std::ostringstream outs;
        $self->print(outs);

        return PyString_FromString(outs.str().c_str());
    }
}

%include "wali_src/Source/wali/wpds/ewpds/EWPDS.hpp"
%include "wali_src/Source/wali/wpds/fwpds/FWPDS.hpp"

%extend wali::wpds::ewpds::EWPDS {
    PyObject* __str__() {
        std::ostringstream outs;
        $self->print(outs);

        return PyString_FromString(outs.str().c_str());
    }
}



%include "wali_src/Examples/Reach/Reach.hpp"

%include "ConstPropagation/ConstDom.hpp"
